#include <iostream>
#include <gsl/gsl_linalg.h>
#include "math.h"

using namespace std;

// put implementation only (private) interface here
namespace Mylib {
}

double Mylib::GslVectorSum(const gsl_vector & a)
{
  double sum = 0;
  for (size_t i = 0; i < a.size; i++) {
    sum += gsl_vector_get(&a, i);
  }
  return sum;
}

int Mylib::GslVectorSum(const gsl_vector_int & a)
{
  int sum = 0;
  for (size_t i = 0; i < a.size; i++) {
    sum += gsl_vector_int_get(&a, i);
  }
  return sum;
}

void Mylib::GslMatrixSetdiag(gsl_matrix * mP, const gsl_vector & v)
{
  for (size_t i = 0; i < v.size; i++) {
    gsl_matrix_set(mP, i, i, gsl_vector_get(&v, i));
  }
}

void Mylib::GslMatrixSum(gsl_vector * vP, const gsl_matrix & a)
{
  for (size_t i = 0; i < a.size2; i++) {
    gsl_vector_set(vP, i, GslVectorSum(gsl_matrix_const_column(&a, i).vector));
  }
}

void Mylib::GslMatrixRowadd(gsl_matrix * mP, const gsl_vector & v)
{
  for (size_t i = 0; i < mP->size1; i++) {
    gsl_vector_add(&gsl_matrix_row(mP, i).vector, &v);
  }
}

void Mylib::GslMatrixRowsub(gsl_matrix * mP, const gsl_vector & v)
{
  for (size_t i = 0; i < mP->size1; i++) {
    gsl_vector_sub(&gsl_matrix_row(mP, i).vector, &v);
  }
}

void Mylib::GslMatrixColadd(gsl_matrix * mP, const gsl_vector & v)
{
  for (size_t i = 0; i < mP->size2; i++) {
    gsl_vector_add(&gsl_matrix_column(mP, i).vector, &v);
  }
}

void Mylib::GslMatrixColsub(gsl_matrix * mP, const gsl_vector & v)
{
  for (size_t i = 0; i < mP->size2; i++) {
    gsl_vector_sub(&gsl_matrix_column(mP, i).vector, &v);
  }
}

int Mylib::GslVectorIsequal(const gsl_vector & v, const gsl_vector & w)
{
  if (v.size != w.size) {
    return 0;
  }
  for (int i = 0; i < v.size; i++) {
    if (gsl_vector_get(&v, i) != gsl_vector_get(&w, i)) {
      return 0;
    }
  }
  return 1;
}

int Mylib::GslVectorIsequal(const gsl_vector_int & v, const gsl_vector_int & w)
{
  if (v.size != w.size) {
    return 0;
  }
  for (int i = 0; i < v.size; i++) {
    if (gsl_vector_int_get(&v, i) != gsl_vector_int_get(&w, i)) {
      return 0;
    }
  }
  return 1;
}

void Mylib::GslMatrixPrint(const gsl_matrix & m)
{
  for (size_t i = 0; i < m.size1; i++) {
    for (size_t j = 0; j < m.size2; j++) {
      if (j != 0) {
	cout << '\t';
      }
      cout << gsl_matrix_get(&m, i, j);
    }
    cout << endl;
  }
}

void Mylib::GslMatrixPrint(const gsl_matrix_int & m)
{
  for (size_t i = 0; i < m.size1; i++) {
    for (size_t j = 0; j < m.size2; j++) {
      if (j != 0) {
	cout << '\t';
      }
      cout << gsl_matrix_int_get(&m, i, j);
    }
    cout << endl;
  }
}

int Mylib::GslMatrixGtlGraph(graph * gp, vector<node> * nsp, const gsl_matrix_int & struc, int directed)
{
  size_t numnode = struc.size1;
  if (numnode != struc.size2) {
    return My_ERROR;
  }
  gp->clear();
  if (directed == Gtl_DIRECTED) {
    gp->make_directed();
  } else {
    gp->make_undirected();
  }
  for (size_t i = 0; i < numnode; i++) {
    nsp->push_back(gp->new_node());
  }
  for (size_t i = 0; i < numnode; i++) {
    for (size_t j = 0; j < numnode; j++) {
      if (gsl_matrix_int_get(&struc, i, j) == 1) {
	gp->new_edge((*nsp)[i], (*nsp)[j]);
      }
    }
  }
  return My_OK;
}

double Mylib::GslMatrixDet(gsl_matrix m)
{
  int sign;
  size_t msize = m.size1;
  if (msize != m.size2) {
    cerr << "Error: determinant undefined for non-square matrix !" << endl;
    exit(1);
  }
  gsl_permutation * permP = gsl_permutation_alloc(msize);
  gsl_linalg_LU_decomp(&m, permP, &sign);
  double det = gsl_linalg_LU_det(&m, sign);
  gsl_permutation_free(permP);
  return det;
}

double Mylib::GslMatrixLndet(int * signP, gsl_matrix m)
{
  size_t msize = m.size1;
  if (msize != m.size2) {
    cerr << "Error: determinant undefined for non-square matrix !" << endl;
    exit(1);
  }
  gsl_permutation * permP = gsl_permutation_alloc(msize);
  gsl_linalg_LU_decomp(&m, permP, signP);
  *signP = gsl_linalg_LU_sgndet(&m, *signP);
  double lnabsdet = gsl_linalg_LU_lndet(&m);
  gsl_permutation_free(permP);
  return lnabsdet;
}

int Mylib::GslMatrixSubmatrix(gsl_matrix * subP, const gsl_matrix & m, const gsl_vector_int & rowind, const gsl_vector_int & colind)
{
  int row = 0;
  for (size_t i = 0; i < rowind.size; i++) {
    if (gsl_vector_int_get(&rowind, i) == 1) {
      int col = 0;
      for (size_t j = 0; j < colind.size; j++) {
	if (gsl_vector_int_get(&colind, j) == 1) {
	  gsl_matrix_set(subP, row, col, gsl_matrix_get(&m, i, j));
	  col++;
	}
      }
      row++;
    }
  }
}
