#ifndef _MYCLIB_GSL_H
#define _MYCLIB_GSL_H

#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <GTL/graph.h>

// put user (public) interface here
namespace Mylib {
  double GslVectorSum(const gsl_vector & a); // sum of all the elements in 'a'
  int GslVectorSum(const gsl_vector_int & a); // sum of all the elements in 'a'
  void GslMatrixSum(gsl_vector * vP, const gsl_matrix & a); // vP is column sum of 'a'
  void GslMatrixSetdiag(gsl_matrix * mP, const gsl_vector & v); // set 'v' be the diagonal of 'mP'. Rest of 'mP' remained UNCHANGED
  void GslMatrixRowadd(gsl_matrix * mP, const gsl_vector & v); // add 'v' to each row of 'mP'
  void GslMatrixRowsub(gsl_matrix * mP, const gsl_vector & v); // subtract 'v' from each row of 'mP'
  void GslMatrixColadd(gsl_matrix * mP, const gsl_vector & v); // add 'v' to each column of 'mP'
  void GslMatrixColsub(gsl_matrix * mP, const gsl_vector & v); // subtract 'v' from each column of 'mP'
  int GslVectorIsequal(const gsl_vector & v, const gsl_vector & w); // return 1 if v == w, 0 otherwise
  int GslVectorIsequal(const gsl_vector_int & v, const gsl_vector_int & w); // return 1 if v == w, 0 otherwise
  void GslMatrixPrint(const gsl_matrix & m); // tab-delimited
  void GslMatrixPrint(const gsl_matrix_int & m); // tab-delimited
  // construct GTL graph 'gp' based on Gsl matrix 'struc', either directed or undirected depending on 'directed'
  // If directed, row index is the parent of column index
  // nsp is all nodes in gp that corresponds to the order in struc
  int GslMatrixGtlGraph(graph * gp, vector<node> * nsp, const gsl_matrix_int & struc, int directed);
  double GslMatrixDet(gsl_matrix m); // Compute the determinant
  double GslMatrixLndet(int * signP, gsl_matrix m); // Compute the log of the determinant
  // take the rows based on 1-element of rowind and columns based on 1-element of colind from 'm', and put into subP
  int GslMatrixSubmatrix(gsl_matrix * subP, const gsl_matrix & m, const gsl_vector_int & rowind, const gsl_vector_int & colind);
  enum {
    Gtl_UNDIRECTED = 0,
    Gtl_DIRECTED = 1
  };
  enum {
    My_OK = 0,
    My_ERROR = 1
  };
}

#endif // _MYCLIB_GSL_H
