#include <iostream>
#include <cmath>
#include "math.h"

using namespace std;

// put implementation only (private) interface here
namespace Mylib {
}

double Mylib::DotProd(const vector<double> & d1, const vector<double> & d2)
{
  int i;
  double ret = 0;
  for (i = 0; i < d1.size(); i++) {
    ret += d1[i] * d2[i];
  }
  return ret;
}

double Mylib::Mean(const vector<double> & vs)
{
  int i;
  double sum = 0;
  for (i = 0; i < vs.size(); i++) {
    sum += vs[i];
  }
  return sum / vs.size();
}

double Mylib::Sqsum(const vector<double> & vs, double mean)
{
  int i;
  double sum = 0, diff;
  for (i = 0; i < vs.size(); i++) {
    diff = vs[i] - mean;
    sum += diff * diff;
  }
  return sum;
}

double Mylib::Var(const vector<double> & vs, double mean)
{
  return Sqsum(vs, mean) / vs.size();
}

void Mylib::NormalizeL1(vector<double> & vs)
{
  double sum = 0;
  int i;
  for (i = 0; i < vs.size(); i++) {
    sum += vs[i];
  }
  if (sum == 0) {
    cerr << "Error: 0 sum" << endl;
    assert(false);
  }
  for (i = 0; i < vs.size(); i++) {
    vs[i] /= sum;
  }
}

double Mylib::LogSum(const vector<double> & vs)
{
  int i;
  double max = vs[0], sum = 0.0;
  for (i = 1; i < vs.size(); i++) {
    if (max < vs[i]) {
      max = vs[i];
    }
  }
  for (i = 0; i < vs.size(); i++) {
    sum += exp(vs[i] - max);
  }
  return log(sum) + max;
}

void Mylib::SortMerge(vector<double> & vs, const vector<double> & v1s, const vector<double> & v2s)
{
  int i1 = 0, i2 = 0;
  vs.clear();
  while (i1 < v1s.size() && i2 < v2s.size()) {
    if (v1s[i1] < v2s[i2]) {
      vs.push_back(v1s[i1]);
      i1++;
    } else if (v1s[i1] > v2s[i2]) {
      vs.push_back(v2s[i2]);
      i2++;
    } else {
      vs.push_back(v1s[i1]);
      i1++;
      i2++;
    }
  }
  for (; i1 < v1s.size(); i1++) {
    vs.push_back(v1s[i1]);
  }
  for (; i2 < v2s.size(); i2++) {
    vs.push_back(v2s[i2]);
  }
}
